#! /usr/bin/env python

# demonstrator.py
# Copyright 2012 Roger Marsh
# Licence: See LICENCE (BSD licence)

"""Demonstrate chess performance calculation by iteration

demonstrator.py does many of the things that hampshire.py did but has the
simpler purpose of just showing the results of a calculation.

"""

if __name__=='__main__':

    # If module not loaded from Python site-packages put the folder
    # containing chessrank at front of sys.path on the assumption all the
    # sibling packages are there too.
    # __file__ not in dir() if run from idle
    if '__file__' in dir():

        import sys
        import os

        sp = sys.path[-1].replace('\\\\', '\\')
        packageroot = os.path.dirname(
            os.path.dirname(os.path.abspath(__file__)))
        if sp != packageroot:
            sys.path.insert(0, packageroot)
        del sp, packageroot

        del os
        del sys

    import Tkinter, tkMessageBox, tkFileDialog
    
    from ECFresults import ResultFolder
    from grade import GradeAnalysis
    import rmstats

    class Display(object):

        def unchanging(self, perf):
            if len(perf) < 4:
                # otherwise the output statement will fail
                # implication is less than 3 iterations - nowhere near enough
                return True
            if str(perf[-1]) != str(perf[-2]):
                return False
            if str(perf[-1]) != str(perf[-3]):
                return False
            return True
        
        def create(self, folder):
            count = sb.get()
            if not count.isdigit():
                tkMessageBox.showinfo(
                    message=''.join(
                        ('Number of iterations must be a number\n\n',
                         str(count),
                         ' is not allowed.',
                         )),
                    title='Open',
                    )
                return
            count = int(count)
            f1 = ResultFolder(folder=folder)
            a = GradeAnalysis(folder=f1)
            a.DoIterations(count=count)
            t = [''.join(('ECF submissions folder: ', folder))]
            t.append('')
            t.append(''.join(
                ('Note that the PINs in the submission files are used to ',
                 'identify players rather than the ECF grading codes.\n\n',
                 'It is assumed that same PIN means same player, and ',
                 'different PIN means different player, across all ',
                 'submission files.  This may not be true.',
                 )))
            if count < 3:
                t.append('')
                t.append('')
                t.append(''.join(
                    ('Less than three iterations done.  All performances are ',
                     'assumed to be still changing at each iteration.',
                     )))
            t.append('')
            t.append('')
            t.append('Summary')
            t.append('')
            t.append(''.join((str(count), ' iterations')))
            t.append('')
            if len(a.grades) == 0:
                tkMessageBox.showinfo(
                    message=''.join(
                        ('Nothing has been calculated.\n\n',
                         'Were there any submission files in folder?',
                         )),
                    title=folder,
                    )
                return
            s = a.GetGradeStatistics()
            for k in sorted(s):
                t.append('\t\t\t\t'.join((k, str(s[k]))))
            t.append('')
            t.append('PIN, Games played, Score, Predicted Score, Performance')
            t.append('')
            pins = sorted(a.predictedscores)
            u = {}
            changing = 0
            for p in pins:
                u[p] = self.unchanging(a.performances[p])
                if not u[p]:
                    if count < 3:
                        f = ''
                    else:
                        changing += 1
                        f = '\t\t\t*****'
                else:
                    f = ''
                t.append(''.join(
                    (str(p), '\t',
                     str(a.games.get(p)), '\t',
                     str(a.scores[p]), '\t',
                     str(a.predictedscores[p]), '\t\t\t',
                     str(a.performances[p][-1]),
                     f,
                     )))
            t.append('')
            if changing == 0:
                if count < 3:
                    t.append(' '.join(
                        ('Less than three iterations.  All performances',
                         'assumed to be changing at each iteration.')))
                else:
                    t.append(' '.join(
                        ('No performances changing after',
                         str(count),
                         'iterations')))
                    t.append('(Precision is 10 decimal places)')
            else:
                t.append(' '.join(
                    (str(changing),
                     'performances changing after',
                     str(count),
                     'iterations')))
                t.append('')
                t.append('Most recent iterations for changing performances')
                t.append('(Those marked with ***** above)')
                t.append('')
                for p in pins:
                    if not u[p]:
                        t.append(''.join(
                            (str(p), '\t',
                             str(a.performances[p][-3]), '\t\t\t',
                             str(a.performances[p][-2]), '\t\t',
                             str(a.performances[p][-1]),
                             )))
            report.delete('1.0', Tkinter.END)
            report.insert(Tkinter.END, '\n'.join(t))
        
    def file_open():
        if not sb.get().isdigit():
            tkMessageBox.showinfo(
                message=''.join(
                    ('Number of iterations must be a number\n\n',
                     str(sb.get()),
                     ' is not allowed.',
                     )),
                title='Open',
                )
            return
        folder = tkFileDialog.askdirectory(
            title='Select folder containing results submissions files',
            initialdir='~',
            mustexist=Tkinter.TRUE)
        if folder:
            display.create(folder)
    
    def quit_edit():
        title='Quit'
        if tkMessageBox.askokcancel(
            master=ser,
            message='Please confirm Quit action',
            title=title):
            ser.winfo_toplevel().destroy()

    display = Display()
    ser = Tkinter.Tk()
    ser.wm_title('Iteration Demonstration')
    sb = Tkinter.Spinbox(
        master=ser, from_=10, to=1000, increment=10)
    menubar = Tkinter.Menu(master=ser)
    menu1 = Tkinter.Menu(menubar, name='file', tearoff=False)
    menubar.add_cascade(label='File', menu=menu1, underline=0)
    menu1.add_command(
        label='Open',
        underline=0,
        command=file_open)
    menu1.add_command(
        label='Quit',
        underline=0,
        command=quit_edit)
    ser.configure(menu=menubar)
    sb.pack()
    report = Tkinter.Text(master=ser, wrap='word')
    scrollbar = Tkinter.Scrollbar(
        master=ser, orient=Tkinter.VERTICAL, command=report.yview)
    report.configure(yscrollcommand=scrollbar.set)
    scrollbar.pack(side=Tkinter.RIGHT, fill=Tkinter.Y)
    report.pack(
        side=Tkinter.LEFT, fill=Tkinter.BOTH, expand=Tkinter.TRUE)
    ser.mainloop()
    del ser
